@@ -17,7 +17,7 @@ func TestMigrationsIsolated(t *testing.T) {
 			&OpRawSQL{
 				Up: `foo`,
 			},
-			&OpRenameColumn{},
+			&OpCreateTable{Name: "foo"},
 		},
 	}
 
@@ -38,3 +38,18 @@ func TestMigrationsIsolatedValid(t *testing.T) {
 	err := migration.Validate(context.TODO(), schema.New())
 	assert.NoError(t, err)
 }
+
+func TestOnCompleteSQLMigrationsAreNotIsolated(t *testing.T) {
+	migration := Migration{
+		Name: "sql",
+		Operations: Operations{
+			&OpRawSQL{
+				Up:         `foo`,
+				OnComplete: true,
+			},
+			&OpCreateTable{Name: "foo"},
+		},
+	}
+	err := migration.Validate(context.TODO(), schema.New())
+	assert.NoError(t, err)
+}
