@@ -9,6 +9,7 @@ import (
 	"strings"
 
 	"github.com/lib/pq"
+
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -23,19 +24,19 @@ func (o *OpAddColumn) Start(ctx context.Context, conn db.DB, latestSchema string
 	}
 
 	if o.Column.Comment != nil {
-		if err := addCommentToColumn(ctx, conn, o.Table, TemporaryName(o.Column.Name), o.Column.Comment); err != nil {
+		if err := addCommentToColumn(ctx, conn, table.Name, TemporaryName(o.Column.Name), o.Column.Comment); err != nil {
 			return nil, fmt.Errorf("failed to add comment to column: %w", err)
 		}
 	}
 
 	if !o.Column.IsNullable() && o.Column.Default == nil {
-		if err := addNotNullConstraint(ctx, conn, o.Table, o.Column.Name, TemporaryName(o.Column.Name)); err != nil {
+		if err := addNotNullConstraint(ctx, conn, table.Name, o.Column.Name, TemporaryName(o.Column.Name)); err != nil {
 			return nil, fmt.Errorf("failed to add not null constraint: %w", err)
 		}
 	}
 
 	if o.Column.Check != nil {
-		if err := o.addCheckConstraint(ctx, conn); err != nil {
+		if err := o.addCheckConstraint(ctx, table.Name, conn); err != nil {
 			return nil, fmt.Errorf("failed to add check constraint: %w", err)
 		}
 	}
@@ -118,7 +119,7 @@ func (o *OpAddColumn) Complete(ctx context.Context, conn db.DB, tr SQLTransforme
 	return err
 }
 
-func (o *OpAddColumn) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
+func (o *OpAddColumn) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
 	tempName := TemporaryName(o.Column.Name)
 
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
@@ -134,6 +135,10 @@ func (o *OpAddColumn) Rollback(ctx context.Context, conn db.DB, tr SQLTransforme
 }
 
 func (o *OpAddColumn) Validate(ctx context.Context, s *schema.Schema) error {
+	if err := ValidateIdentifierLength(o.Column.Name); err != nil {
+		return err
+	}
+
 	table := s.GetTable(o.Table)
 	if table == nil {
 		return TableDoesNotExistError{Name: o.Table}
@@ -171,7 +176,7 @@ func (o *OpAddColumn) Validate(ctx context.Context, s *schema.Schema) error {
 		}
 	}
 
-	if !o.Column.IsNullable() && o.Column.Default == nil && o.Up == "" {
+	if !o.Column.IsNullable() && o.Column.Default == nil && o.Up == "" && !o.Column.HasImplicitDefault() {
 		return FieldRequiredError{Name: "up"}
 	}
 
@@ -179,6 +184,12 @@ func (o *OpAddColumn) Validate(ctx context.Context, s *schema.Schema) error {
 		return errors.New("adding primary key columns is not supported")
 	}
 
+	// Update the schema to ensure that the new column is visible to validation of
+	// subsequent operations.
+	table.AddColumn(o.Column.Name, schema.Column{
+		Name: TemporaryName(o.Column.Name),
+	})
+
 	return nil
 }
 
@@ -191,7 +202,7 @@ func addColumn(ctx context.Context, conn db.DB, o OpAddColumn, t *schema.Table,
 	//   on migration completion
 	// This is to avoid unnecessary exclusive table locks.
 	if !o.Column.IsNullable() && o.Column.Default == nil {
-		o.Column.Nullable = ptr(true)
+		o.Column.Nullable = true
 	}
 
 	// Don't add a column with a CHECK constraint directly.
@@ -203,7 +214,8 @@ func addColumn(ctx context.Context, conn db.DB, o OpAddColumn, t *schema.Table,
 	o.Column.Check = nil
 
 	o.Column.Name = TemporaryName(o.Column.Name)
-	colSQL, err := ColumnToSQL(o.Column, tr)
+	columnWriter := ColumnSQLWriter{WithPK: true, Transformer: tr}
+	colSQL, err := columnWriter.Write(o.Column)
 	if err != nil {
 		return err
 	}
@@ -225,11 +237,11 @@ func addNotNullConstraint(ctx context.Context, conn db.DB, table, column, physic
 	return err
 }
 
-func (o *OpAddColumn) addCheckConstraint(ctx context.Context, conn db.DB) error {
+func (o *OpAddColumn) addCheckConstraint(ctx context.Context, tableName string, conn db.DB) error {
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s ADD CONSTRAINT %s CHECK (%s) NOT VALID",
-		pq.QuoteIdentifier(o.Table),
+		pq.QuoteIdentifier(tableName),
 		pq.QuoteIdentifier(o.Column.Check.Name),
-		rewriteCheckExpression(o.Column.Check.Constraint, o.Column.Name, TemporaryName(o.Column.Name)),
+		rewriteCheckExpression(o.Column.Check.Constraint, o.Column.Name),
 	))
 	return err
 }
@@ -243,3 +255,51 @@ func NotNullConstraintName(columnName string) string {
 func IsNotNullConstraintName(name string) bool {
 	return strings.HasPrefix(name, "_pgroll_check_not_null_")
 }
+
+// ColumnSQLWriter writes a column to SQL
+// It can optionally include the primary key constraint
+// When creating a table, the primary key constraint is not added to the column definition
+type ColumnSQLWriter struct {
+	WithPK      bool
+	Transformer SQLTransformer
+}
+
+func (w ColumnSQLWriter) Write(col Column) (string, error) {
+	sql := fmt.Sprintf("%s %s", pq.QuoteIdentifier(col.Name), col.Type)
+
+	if w.WithPK && col.IsPrimaryKey() {
+		sql += " PRIMARY KEY"
+	}
+
+	if col.IsUnique() {
+		sql += " UNIQUE"
+	}
+	if !col.IsNullable() {
+		sql += " NOT NULL"
+	}
+	if col.Default != nil {
+		d, err := w.Transformer.TransformSQL(*col.Default)
+		if err != nil {
+			return "", err
+		}
+		sql += fmt.Sprintf(" DEFAULT %s", d)
+	}
+	if col.References != nil {
+		onDelete := string(ForeignKeyReferenceOnDeleteNOACTION)
+		if col.References.OnDelete != "" {
+			onDelete = strings.ToUpper(string(col.References.OnDelete))
+		}
+
+		sql += fmt.Sprintf(" CONSTRAINT %s REFERENCES %s(%s) ON DELETE %s",
+			pq.QuoteIdentifier(col.References.Name),
+			pq.QuoteIdentifier(col.References.Table),
+			pq.QuoteIdentifier(col.References.Column),
+			onDelete)
+	}
+	if col.Check != nil {
+		sql += fmt.Sprintf(" CONSTRAINT %s CHECK (%s)",
+			pq.QuoteIdentifier(col.Check.Name),
+			col.Check.Constraint)
+	}
+	return sql, nil
+}
