@@ -12,9 +12,11 @@ import (
 	"testing"
 
 	"github.com/lib/pq"
+	"github.com/stretchr/testify/assert"
+
+	"github.com/xataio/pgroll/internal/testutils"
 	"github.com/xataio/pgroll/pkg/migrations"
 	"github.com/xataio/pgroll/pkg/roll"
-	"github.com/xataio/pgroll/pkg/testutils"
 )
 
 type TestCase struct {
@@ -178,6 +180,13 @@ func ColumnMustNotHaveComment(t *testing.T, db *sql.DB, schema, table, column st
 	}
 }
 
+func ColumnMustBePK(t *testing.T, db *sql.DB, schema, table, column string) {
+	t.Helper()
+	if !columnMustBePK(t, db, schema, table, column) {
+		t.Fatalf("Expected column %q to be primary key", column)
+	}
+}
+
 func TableMustHaveComment(t *testing.T, db *sql.DB, schema, table, expectedComment string) {
 	t.Helper()
 	if !tableHasComment(t, db, schema, table, expectedComment) {
@@ -296,6 +305,27 @@ func indexExists(t *testing.T, db *sql.DB, schema, table, index string) bool {
 	return exists
 }
 
+func CheckIndexDefinition(t *testing.T, db *sql.DB, schema, table, index, expectedDefinition string) {
+	t.Helper()
+
+	var actualDef string
+	err := db.QueryRow(`
+      SELECT indexdef
+      FROM pg_indexes
+      WHERE schemaname = $1
+      AND tablename = $2
+      AND indexname = $3
+    `,
+		schema, table, index).Scan(&actualDef)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if expectedDefinition != actualDef {
+		t.Fatalf("Expected index %q to have definition %q, got %q", index, expectedDefinition, actualDef)
+	}
+}
+
 func checkConstraintExists(t *testing.T, db *sql.DB, schema, table, constraint string) bool {
 	t.Helper()
 
@@ -526,6 +556,28 @@ func columnHasComment(t *testing.T, db *sql.DB, schema, table, column string, ex
 	return actualComment != nil && *expectedComment == *actualComment
 }
 
+func columnMustBePK(t *testing.T, db *sql.DB, schema, table, column string) bool {
+	t.Helper()
+
+	var exists bool
+	err := db.QueryRow(fmt.Sprintf(`
+    SELECT EXISTS (
+	  SELECT a.attname
+      FROM   pg_index i
+      JOIN   pg_attribute a ON a.attrelid = i.indrelid
+                      AND a.attnum = ANY(i.indkey)
+      WHERE  i.indrelid = %[1]s::regclass AND i.indisprimary AND a.attname = %[2]s
+    )`,
+		pq.QuoteLiteral(fmt.Sprintf("%s.%s", schema, table)),
+		pq.QuoteLiteral(column)),
+	).Scan(&exists)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	return exists
+}
+
 func tableHasComment(t *testing.T, db *sql.DB, schema, table, expectedComment string) bool {
 	t.Helper()
 
@@ -690,10 +742,37 @@ func MustSelect(t *testing.T, db *sql.DB, schema, version, table string) []map[s
 
 		res = append(res, row)
 	}
+	assert.NoError(t, q.Err())
 
 	return res
 }
 
+// TableMustBeCleanedUp asserts that the `columns` on `table` in `schema` have
+// been cleaned up after migration rollback or completion. This means:
+// - The temporary columns should not exist on the underlying table.
+// - The up functions for the columns no longer exist.
+// - The down functions for the columns no longer exist.
+// - The up triggers for the columns no longer exist.
+// - The down triggers for the columns no longer exist.
+func TableMustBeCleanedUp(t *testing.T, db *sql.DB, schema, table string, columns ...string) {
+	t.Helper()
+
+	for _, column := range columns {
+		// The temporary column should not exist on the underlying table.
+		ColumnMustNotExist(t, db, schema, table, migrations.TemporaryName(column))
+
+		// The up function for the column no longer exists.
+		FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName(table, column))
+		// The down function for the column no longer exists.
+		FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName(table, migrations.TemporaryName(column)))
+
+		// The up trigger for the column no longer exists.
+		TriggerMustNotExist(t, db, schema, table, migrations.TriggerName(table, column))
+		// The down trigger for the column no longer exists.
+		TriggerMustNotExist(t, db, schema, table, migrations.TriggerName(table, migrations.TemporaryName(column)))
+	}
+}
+
 func mustSetSearchPath(t *testing.T, db *sql.DB, schema string) {
 	t.Helper()
 
