@@ -8,6 +8,7 @@ import (
 	"strings"
 
 	"github.com/lib/pq"
+
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -15,11 +16,28 @@ import (
 var _ Operation = (*OpCreateIndex)(nil)
 
 func (o *OpCreateIndex) Start(ctx context.Context, conn db.DB, latestSchema string, tr SQLTransformer, s *schema.Schema, cbs ...CallbackFn) (*schema.Table, error) {
+	table := s.GetTable(o.Table)
+
 	// create index concurrently
-	stmt := fmt.Sprintf("CREATE INDEX CONCURRENTLY %s ON %s (%s)",
+	stmtFmt := "CREATE INDEX CONCURRENTLY %s ON %s"
+	if o.Unique != nil && *o.Unique {
+		stmtFmt = "CREATE UNIQUE INDEX CONCURRENTLY %s ON %s"
+	}
+	stmt := fmt.Sprintf(stmtFmt,
 		pq.QuoteIdentifier(o.Name),
-		pq.QuoteIdentifier(o.Table),
-		strings.Join(quoteColumnNames(o.Columns), ", "))
+		pq.QuoteIdentifier(table.Name))
+
+	if o.Method != nil {
+		stmt += fmt.Sprintf(" USING %s", string(*o.Method))
+	}
+
+	stmt += fmt.Sprintf(" (%s)", strings.Join(
+		quoteColumnNames(table.PhysicalColumnNamesFor(o.Columns...)), ", "),
+	)
+
+	if o.StorageParameters != nil {
+		stmt += fmt.Sprintf(" WITH (%s)", *o.StorageParameters)
+	}
 
 	if o.Predicate != nil {
 		stmt += fmt.Sprintf(" WHERE %s", *o.Predicate)
@@ -34,7 +52,7 @@ func (o *OpCreateIndex) Complete(ctx context.Context, conn db.DB, tr SQLTransfor
 	return nil
 }
 
-func (o *OpCreateIndex) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
+func (o *OpCreateIndex) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
 	// drop the index concurrently
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("DROP INDEX CONCURRENTLY IF EXISTS %s",
 		pq.QuoteIdentifier(o.Name)))
@@ -47,6 +65,10 @@ func (o *OpCreateIndex) Validate(ctx context.Context, s *schema.Schema) error {
 		return FieldRequiredError{Name: "name"}
 	}
 
+	if err := ValidateIdentifierLength(o.Name); err != nil {
+		return err
+	}
+
 	table := s.GetTable(o.Table)
 	if table == nil {
 		return TableDoesNotExistError{Name: o.Table}
