@@ -8,6 +8,7 @@ import (
 	"strings"
 
 	"github.com/lib/pq"
+
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -46,17 +47,8 @@ func (o *OpCreateTable) Start(ctx context.Context, conn db.DB, latestSchema stri
 		}
 	}
 
-	columns := make(map[string]schema.Column, len(o.Columns))
-	for _, col := range o.Columns {
-		columns[col.Name] = schema.Column{
-			Name: col.Name,
-		}
-	}
-
-	s.AddTable(o.Name, schema.Table{
-		Name:    tempName,
-		Columns: columns,
-	})
+	// Update the in-memory schema representation with the new table
+	o.updateSchema(s)
 
 	return nil, nil
 }
@@ -69,7 +61,7 @@ func (o *OpCreateTable) Complete(ctx context.Context, conn db.DB, tr SQLTransfor
 	return err
 }
 
-func (o *OpCreateTable) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
+func (o *OpCreateTable) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
 	tempName := TemporaryName(o.Name)
 
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("DROP TABLE IF EXISTS %s",
@@ -78,12 +70,20 @@ func (o *OpCreateTable) Rollback(ctx context.Context, conn db.DB, tr SQLTransfor
 }
 
 func (o *OpCreateTable) Validate(ctx context.Context, s *schema.Schema) error {
+	if err := ValidateIdentifierLength(o.Name); err != nil {
+		return err
+	}
+
 	table := s.GetTable(o.Name)
 	if table != nil {
 		return TableAlreadyExistsError{Name: o.Name}
 	}
 
 	for _, col := range o.Columns {
+		if err := ValidateIdentifierLength(col.Name); err != nil {
+			return fmt.Errorf("invalid column: %w", err)
+		}
+
 		// Ensure that any foreign key references are valid, ie. the referenced
 		// table and column exist.
 		if col.References != nil {
@@ -108,60 +108,51 @@ func (o *OpCreateTable) Validate(ctx context.Context, s *schema.Schema) error {
 		}
 	}
 
+	// Update the schema to ensure that the new table is visible to validation of
+	// subsequent operations.
+	o.updateSchema(s)
+
 	return nil
 }
 
+// updateSchema updates the in-memory schema representation with the details of
+// the new table.
+func (o *OpCreateTable) updateSchema(s *schema.Schema) *schema.Schema {
+	columns := make(map[string]schema.Column, len(o.Columns))
+	for _, col := range o.Columns {
+		columns[col.Name] = schema.Column{
+			Name: col.Name,
+		}
+	}
+	s.AddTable(o.Name, schema.Table{
+		Name:    TemporaryName(o.Name),
+		Columns: columns,
+	})
+
+	return s
+}
+
 func columnsToSQL(cols []Column, tr SQLTransformer) (string, error) {
 	var sql string
+	var primaryKeys []string
+	columnWriter := ColumnSQLWriter{WithPK: false, Transformer: tr}
 	for i, col := range cols {
 		if i > 0 {
 			sql += ", "
 		}
-		colSQL, err := ColumnToSQL(col, tr)
+		colSQL, err := columnWriter.Write(col)
 		if err != nil {
 			return "", err
 		}
 		sql += colSQL
-	}
-	return sql, nil
-}
-
-// ColumnToSQL generates the SQL for a column definition.
-func ColumnToSQL(col Column, tr SQLTransformer) (string, error) {
-	sql := fmt.Sprintf("%s %s", pq.QuoteIdentifier(col.Name), col.Type)
 
-	if col.IsPrimaryKey() {
-		sql += " PRIMARY KEY"
-	}
-	if col.IsUnique() {
-		sql += " UNIQUE"
-	}
-	if !col.IsNullable() {
-		sql += " NOT NULL"
-	}
-	if col.Default != nil {
-		d, err := tr.TransformSQL(*col.Default)
-		if err != nil {
-			return "", err
+		if col.IsPrimaryKey() {
+			primaryKeys = append(primaryKeys, pq.QuoteIdentifier(col.Name))
 		}
-		sql += fmt.Sprintf(" DEFAULT %s", d)
 	}
-	if col.References != nil {
-		onDelete := "NO ACTION"
-		if col.References.OnDelete != "" {
-			onDelete = strings.ToUpper(string(col.References.OnDelete))
-		}
 
-		sql += fmt.Sprintf(" CONSTRAINT %s REFERENCES %s(%s) ON DELETE %s",
-			pq.QuoteIdentifier(col.References.Name),
-			pq.QuoteIdentifier(col.References.Table),
-			pq.QuoteIdentifier(col.References.Column),
-			onDelete)
-	}
-	if col.Check != nil {
-		sql += fmt.Sprintf(" CONSTRAINT %s CHECK (%s)",
-			pq.QuoteIdentifier(col.Check.Name),
-			col.Check.Constraint)
+	if len(primaryKeys) > 0 {
+		sql += fmt.Sprintf(", PRIMARY KEY (%s)", strings.Join(primaryKeys, ", "))
 	}
 	return sql, nil
 }
