@@ -7,6 +7,7 @@ import (
 	"fmt"
 
 	"github.com/lib/pq"
+
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -15,7 +16,10 @@ var _ Operation = (*OpDropConstraint)(nil)
 
 func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema string, tr SQLTransformer, s *schema.Schema, cbs ...CallbackFn) (*schema.Table, error) {
 	table := s.GetTable(o.Table)
-	column := table.GetColumn(o.Column)
+
+	// By this point Validate() should have run which ensures the constraint exists and that we only have
+	// one column associated with it.
+	column := table.GetColumn(table.GetConstraintColumns(o.Name)[0])
 
 	// Create a copy of the column on the underlying table.
 	d := NewColumnDuplicator(conn, table, column).WithoutConstraint(o.Name)
@@ -25,14 +29,14 @@ func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema s
 
 	// Add a trigger to copy values from the old column to the new, rewriting values using the `up` SQL.
 	err := createTrigger(ctx, conn, tr, triggerConfig{
-		Name:           TriggerName(o.Table, o.Column),
+		Name:           TriggerName(o.Table, column.Name),
 		Direction:      TriggerDirectionUp,
 		Columns:        table.Columns,
 		SchemaName:     s.Name,
 		LatestSchema:   latestSchema,
 		TableName:      o.Table,
-		PhysicalColumn: TemporaryName(o.Column),
-		SQL:            o.upSQL(),
+		PhysicalColumn: TemporaryName(column.Name),
+		SQL:            o.upSQL(column.Name),
 	})
 	if err != nil {
 		return nil, fmt.Errorf("failed to create up trigger: %w", err)
@@ -41,19 +45,19 @@ func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema s
 	// Add the new column to the internal schema representation. This is done
 	// here, before creation of the down trigger, so that the trigger can declare
 	// a variable for the new column.
-	table.AddColumn(o.Column, schema.Column{
-		Name: TemporaryName(o.Column),
+	table.AddColumn(column.Name, schema.Column{
+		Name: TemporaryName(column.Name),
 	})
 
 	// Add a trigger to copy values from the new column to the old, rewriting values using the `down` SQL.
 	err = createTrigger(ctx, conn, tr, triggerConfig{
-		Name:           TriggerName(o.Table, TemporaryName(o.Column)),
+		Name:           TriggerName(o.Table, TemporaryName(column.Name)),
 		Direction:      TriggerDirectionDown,
 		Columns:        table.Columns,
 		SchemaName:     s.Name,
 		LatestSchema:   latestSchema,
 		TableName:      o.Table,
-		PhysicalColumn: o.Column,
+		PhysicalColumn: column.Name,
 		SQL:            o.Down,
 	})
 	if err != nil {
@@ -63,16 +67,20 @@ func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema s
 }
 
 func (o *OpDropConstraint) Complete(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
+	// We have already validated that there is single column related to this constraint.
+	table := s.GetTable(o.Table)
+	column := table.GetColumn(table.GetConstraintColumns(o.Name)[0])
+
 	// Remove the up function and trigger
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, o.Column))))
+		pq.QuoteIdentifier(TriggerFunctionName(o.Table, column.Name))))
 	if err != nil {
 		return err
 	}
 
 	// Remove the down function and trigger
 	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, TemporaryName(o.Column)))))
+		pq.QuoteIdentifier(TriggerFunctionName(o.Table, TemporaryName(column.Name)))))
 	if err != nil {
 		return err
 	}
@@ -80,14 +88,12 @@ func (o *OpDropConstraint) Complete(ctx context.Context, conn db.DB, tr SQLTrans
 	// Drop the old column
 	_, err = conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
 		pq.QuoteIdentifier(o.Table),
-		pq.QuoteIdentifier(o.Column)))
+		pq.QuoteIdentifier(column.Name)))
 	if err != nil {
 		return err
 	}
 
 	// Rename the new column to the old column name
-	table := s.GetTable(o.Table)
-	column := table.GetColumn(o.Column)
 	if err := RenameDuplicatedColumn(ctx, conn, table, column); err != nil {
 		return err
 	}
@@ -95,11 +101,15 @@ func (o *OpDropConstraint) Complete(ctx context.Context, conn db.DB, tr SQLTrans
 	return err
 }
 
-func (o *OpDropConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
+func (o *OpDropConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
+	// We have already validated that there is single column related to this constraint.
+	table := s.GetTable(o.Table)
+	column := table.GetColumn(table.GetConstraintColumns(o.Name)[0])
+
 	// Drop the new column
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s DROP COLUMN IF EXISTS %s",
 		pq.QuoteIdentifier(o.Table),
-		pq.QuoteIdentifier(TemporaryName(o.Column)),
+		pq.QuoteIdentifier(TemporaryName(column.Name)),
 	))
 	if err != nil {
 		return err
@@ -107,7 +117,7 @@ func (o *OpDropConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTrans
 
 	// Remove the up function and trigger
 	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, o.Column)),
+		pq.QuoteIdentifier(TriggerFunctionName(o.Table, column.Name)),
 	))
 	if err != nil {
 		return err
@@ -115,7 +125,7 @@ func (o *OpDropConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTrans
 
 	// Remove the down function and trigger
 	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, TemporaryName(o.Column))),
+		pq.QuoteIdentifier(TriggerFunctionName(o.Table, TemporaryName(column.Name))),
 	))
 
 	return err
@@ -127,11 +137,6 @@ func (o *OpDropConstraint) Validate(ctx context.Context, s *schema.Schema) error
 		return TableDoesNotExistError{Name: o.Table}
 	}
 
-	column := table.GetColumn(o.Column)
-	if column == nil {
-		return ColumnDoesNotExistError{Table: o.Table, Name: o.Column}
-	}
-
 	if o.Name == "" {
 		return FieldRequiredError{Name: "name"}
 	}
@@ -140,6 +145,17 @@ func (o *OpDropConstraint) Validate(ctx context.Context, s *schema.Schema) error
 		return ConstraintDoesNotExistError{Table: o.Table, Constraint: o.Name}
 	}
 
+	columns := table.GetConstraintColumns(o.Name)
+
+	// We already know the constraint exists because we checked it earlier so we only need to check the
+	// case where there are multiple columns.
+	if len(columns) > 1 {
+		return MultiColumnConstraintsNotSupportedError{
+			Table:      table.Name,
+			Constraint: o.Name,
+		}
+	}
+
 	if o.Down == "" {
 		return FieldRequiredError{Name: "down"}
 	}
@@ -147,10 +163,10 @@ func (o *OpDropConstraint) Validate(ctx context.Context, s *schema.Schema) error
 	return nil
 }
 
-func (o *OpDropConstraint) upSQL() string {
+func (o *OpDropConstraint) upSQL(column string) string {
 	if o.Up != "" {
 		return o.Up
 	}
 
-	return pq.QuoteIdentifier(o.Column)
+	return pq.QuoteIdentifier(column)
 }
