@@ -6,9 +6,11 @@ import (
 	"database/sql"
 	"testing"
 
+	"github.com/xataio/pgroll/internal/testutils"
+
 	"github.com/stretchr/testify/assert"
+
 	"github.com/xataio/pgroll/pkg/migrations"
-	"github.com/xataio/pgroll/pkg/testutils"
 )
 
 func TestDropNotNull(t *testing.T) {
@@ -27,22 +29,22 @@ func TestDropNotNull(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "username",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "product",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "review",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -55,7 +57,7 @@ func TestDropNotNull(t *testing.T) {
 							Table:    "reviews",
 							Column:   "review",
 							Nullable: ptr(true),
-							Down:     "(SELECT CASE WHEN review IS NULL THEN product || ' is good' ELSE review END)",
+							Down:     "SELECT CASE WHEN review IS NULL THEN product || ' is good' ELSE review END",
 						},
 					},
 				},
@@ -108,22 +110,12 @@ func TestDropNotNull(t *testing.T) {
 				}, rows)
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
-				// The new (temporary) `review` column should not exist on the underlying table.
-				ColumnMustNotExist(t, db, schema, "reviews", migrations.TemporaryName("review"))
-
-				// The up function no longer exists.
-				FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName("reviews", "review"))
-				// The down function no longer exists.
-				FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName("reviews", migrations.TemporaryName("review")))
-
-				// The up trigger no longer exists.
-				TriggerMustNotExist(t, db, schema, "reviews", migrations.TriggerName("reviews", "review"))
-				// The down trigger no longer exists.
-				TriggerMustNotExist(t, db, schema, "reviews", migrations.TriggerName("reviews", migrations.TemporaryName("review")))
+				// The table is cleaned up; temporary columns, trigger functions and triggers no longer exist.
+				TableMustBeCleanedUp(t, db, schema, "reviews", "review")
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
-				// The new (temporary) `review` column should not exist on the underlying table.
-				ColumnMustNotExist(t, db, schema, "reviews", migrations.TemporaryName("review"))
+				// The table is cleaned up; temporary columns, trigger functions and triggers no longer exist.
+				TableMustBeCleanedUp(t, db, schema, "reviews", "review")
 
 				// Writing a NULL review into the `review` column should succeed.
 				MustInsert(t, db, schema, "02_set_nullable", "reviews", map[string]string{
@@ -139,16 +131,6 @@ func TestDropNotNull(t *testing.T) {
 					{"id": 4, "username": "dana", "product": "durian", "review": "delicious"},
 					{"id": 5, "username": "earl", "product": "eggplant", "review": nil},
 				}, rows)
-
-				// The up function no longer exists.
-				FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName("reviews", "review"))
-				// The down function no longer exists.
-				FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName("reviews", migrations.TemporaryName("review")))
-
-				// The up trigger no longer exists.
-				TriggerMustNotExist(t, db, schema, "reviews", migrations.TriggerName("reviews", "review"))
-				// The down trigger no longer exists.
-				TriggerMustNotExist(t, db, schema, "reviews", migrations.TriggerName("reviews", migrations.TemporaryName("review")))
 			},
 		},
 		{
@@ -163,22 +145,22 @@ func TestDropNotNull(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "username",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "product",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "review",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -191,7 +173,7 @@ func TestDropNotNull(t *testing.T) {
 							Table:    "reviews",
 							Column:   "review",
 							Nullable: ptr(true),
-							Down:     "(SELECT CASE WHEN review IS NULL THEN product || ' is good' ELSE review END)",
+							Down:     "SELECT CASE WHEN review IS NULL THEN product || ' is good' ELSE review END",
 							Up:       "review || ' (from the old column)'",
 						},
 					},
@@ -229,12 +211,12 @@ func TestDropNotNull(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -249,17 +231,17 @@ func TestDropNotNull(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "department_id",
 									Type:     "integer",
-									Nullable: ptr(false),
+									Nullable: false,
 									References: &migrations.ForeignKeyReference{
 										Name:   "fk_employee_department",
 										Table:  "departments",
@@ -277,7 +259,7 @@ func TestDropNotNull(t *testing.T) {
 							Table:    "employees",
 							Column:   "department_id",
 							Nullable: ptr(true),
-							Down:     "(SELECT CASE WHEN department_id IS NULL THEN 1 ELSE department_id END)",
+							Down:     "SELECT CASE WHEN department_id IS NULL THEN 1 ELSE department_id END",
 							Up:       "department_id",
 						},
 					},
@@ -306,12 +288,12 @@ func TestDropNotNull(t *testing.T) {
 								{
 									Name: "id",
 									Type: "integer",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 									Default:  ptr("'anonymous'"),
 								},
 							},
@@ -326,7 +308,7 @@ func TestDropNotNull(t *testing.T) {
 							Column:   "name",
 							Nullable: ptr(true),
 							Up:       "name",
-							Down:     "(SELECT CASE WHEN name IS NULL THEN 'anonymous' ELSE name END)",
+							Down:     "SELECT CASE WHEN name IS NULL THEN 'anonymous' ELSE name END",
 						},
 					},
 				},
@@ -371,12 +353,12 @@ func TestDropNotNull(t *testing.T) {
 								{
 									Name: "id",
 									Type: "integer",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 									Check: &migrations.CheckConstraint{
 										Name:       "name_length",
 										Constraint: "length(name) > 3",
@@ -394,7 +376,7 @@ func TestDropNotNull(t *testing.T) {
 							Column:   "name",
 							Nullable: ptr(true),
 							Up:       "name",
-							Down:     "(SELECT CASE WHEN name IS NULL THEN 'anonymous' ELSE name END)",
+							Down:     "SELECT CASE WHEN name IS NULL THEN 'anonymous' ELSE name END",
 						},
 					},
 				},
@@ -428,12 +410,12 @@ func TestDropNotNull(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -459,7 +441,7 @@ func TestDropNotNull(t *testing.T) {
 							Column:   "name",
 							Nullable: ptr(true),
 							Up:       "name",
-							Down:     "(SELECT CASE WHEN name IS NULL THEN 'anonymous' ELSE name END)",
+							Down:     "SELECT CASE WHEN name IS NULL THEN 'anonymous' ELSE name END",
 						},
 					},
 				},
@@ -504,12 +486,12 @@ func TestDropNotNull(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 									Comment:  ptr("the name of the user"),
 								},
 							},
@@ -524,7 +506,7 @@ func TestDropNotNull(t *testing.T) {
 							Column:   "name",
 							Nullable: ptr(true),
 							Up:       "name",
-							Down:     "(SELECT CASE WHEN name IS NULL THEN 'anonymous' ELSE name END)",
+							Down:     "SELECT CASE WHEN name IS NULL THEN 'anonymous' ELSE name END",
 						},
 					},
 				},
@@ -559,12 +541,12 @@ func TestDropNotNullValidation(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -596,12 +578,12 @@ func TestDropNotNullValidation(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(true),
+									Nullable: true,
 								},
 							},
 						},
@@ -615,7 +597,7 @@ func TestDropNotNullValidation(t *testing.T) {
 							Column:   "name",
 							Nullable: ptr(true),
 							Up:       "name",
-							Down:     "(SELECT CASE WHEN name IS NULL THEN 'placeholder' ELSE name END)",
+							Down:     "SELECT CASE WHEN name IS NULL THEN 'placeholder' ELSE name END",
 						},
 					},
 				},
