@@ -7,6 +7,7 @@ import (
 	"fmt"
 
 	"github.com/lib/pq"
+
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -27,7 +28,7 @@ func (o *OpRenameConstraint) Complete(ctx context.Context, conn db.DB, tr SQLTra
 	return err
 }
 
-func (o *OpRenameConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
+func (o *OpRenameConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
 	// no-op
 	return nil
 }
@@ -47,5 +48,9 @@ func (o *OpRenameConstraint) Validate(ctx context.Context, s *schema.Schema) err
 		return ConstraintAlreadyExistsError{Table: o.Table, Constraint: o.To}
 	}
 
+	if err := ValidateIdentifierLength(o.To); err != nil {
+		return err
+	}
+
 	return nil
 }
