@@ -7,6 +7,7 @@ import (
 	"fmt"
 
 	"github.com/lib/pq"
+
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -24,7 +25,7 @@ func (o *OpRenameTable) Complete(ctx context.Context, conn db.DB, tr SQLTransfor
 	return err
 }
 
-func (o *OpRenameTable) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
+func (o *OpRenameTable) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
 	return nil
 }
 
@@ -35,6 +36,9 @@ func (o *OpRenameTable) Validate(ctx context.Context, s *schema.Schema) error {
 	if s.GetTable(o.To) != nil {
 		return TableAlreadyExistsError{Name: o.To}
 	}
+	if err := ValidateIdentifierLength(o.To); err != nil {
+		return err
+	}
 
 	return nil
 }
