@@ -8,6 +8,7 @@ import (
 	"strings"
 
 	"github.com/lib/pq"
+
 	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -45,7 +46,7 @@ func (o *OpSetCheckConstraint) Complete(ctx context.Context, conn db.DB, tr SQLT
 	return nil
 }
 
-func (o *OpSetCheckConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
+func (o *OpSetCheckConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
 	return nil
 }
 
@@ -58,6 +59,18 @@ func (o *OpSetCheckConstraint) Validate(ctx context.Context, s *schema.Schema) e
 		}
 	}
 
+	table := s.GetTable(o.Table)
+	if table == nil {
+		return TableDoesNotExistError{Name: o.Table}
+	}
+
+	if table.ConstraintExists(o.Check.Name) {
+		return ConstraintAlreadyExistsError{
+			Table:      table.Name,
+			Constraint: o.Check.Name,
+		}
+	}
+
 	if o.Up == "" {
 		return FieldRequiredError{Name: "up"}
 	}
@@ -73,7 +86,7 @@ func (o *OpSetCheckConstraint) addCheckConstraint(ctx context.Context, conn db.D
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s ADD CONSTRAINT %s CHECK (%s) NOT VALID",
 		pq.QuoteIdentifier(o.Table),
 		pq.QuoteIdentifier(o.Check.Name),
-		rewriteCheckExpression(o.Check.Constraint, o.Column, TemporaryName(o.Column)),
+		rewriteCheckExpression(o.Check.Constraint, o.Column),
 	))
 
 	return err
@@ -84,6 +97,9 @@ func (o *OpSetCheckConstraint) addCheckConstraint(ctx context.Context, conn db.D
 // On migration start, however, the check is actually applied to the new (temporary)
 // column.
 // This function naively rewrites the check expression to apply to the new column.
-func rewriteCheckExpression(check string, oldColumn, newColumn string) string {
-	return strings.ReplaceAll(check, oldColumn, newColumn)
+func rewriteCheckExpression(check string, columns ...string) string {
+	for _, col := range columns {
+		check = strings.ReplaceAll(check, col, TemporaryName(col))
+	}
+	return check
 }
