@@ -9,6 +9,7 @@ import (
 	"strings"
 
 	"github.com/lib/pq"
+
 	"github.com/xataio/pgroll/pkg/migrations"
 	"github.com/xataio/pgroll/pkg/schema"
 )
@@ -43,12 +44,14 @@ func (m *Roll) StartDDLOperations(ctx context.Context, migration *migrations.Mig
 	}
 
 	// validate migration
-	err = migration.Validate(ctx, newSchema)
-	if err != nil {
-		if err := m.state.Rollback(ctx, m.schema, migration.Name); err != nil {
-			fmt.Printf("failed to rollback migration: %s\n", err)
+	if !m.skipValidation {
+		err = migration.Validate(ctx, newSchema)
+		if err != nil {
+			if err := m.state.Rollback(ctx, m.schema, migration.Name); err != nil {
+				fmt.Printf("failed to rollback migration: %s\n", err)
+			}
+			return nil, fmt.Errorf("migration is invalid: %w", err)
 		}
-		return nil, fmt.Errorf("migration is invalid: %w", err)
 	}
 
 	// run any BeforeStartDDL hooks
@@ -72,6 +75,13 @@ func (m *Roll) StartDDLOperations(ctx context.Context, migration *migrations.Mig
 	}
 	latestSchema := VersionedSchemaName(m.schema, *latestVersion)
 
+	// Reread the latest schema as validation may have updated the schema object
+	// in memory.
+	newSchema, err = m.state.ReadSchema(ctx, m.schema)
+	if err != nil {
+		return nil, fmt.Errorf("unable to read schema: %w", err)
+	}
+
 	// execute operations
 	var tablesToBackfill []*schema.Table
 	for _, op := range migration.Operations {
@@ -155,7 +165,7 @@ func (m *Roll) Complete(ctx context.Context) error {
 	}
 
 	// read the current schema
-	schema, err := m.state.ReadSchema(ctx, m.schema)
+	currentSchema, err := m.state.ReadSchema(ctx, m.schema)
 	if err != nil {
 		return fmt.Errorf("unable to read schema: %w", err)
 	}
@@ -175,7 +185,7 @@ func (m *Roll) Complete(ctx context.Context) error {
 	// execute operations
 	refreshViews := false
 	for _, op := range migration.Operations {
-		err := op.Complete(ctx, m.pgConn, m.sqlTransformer, schema)
+		err := op.Complete(ctx, m.pgConn, m.sqlTransformer, currentSchema)
 		if err != nil {
 			return fmt.Errorf("unable to execute complete operation: %w", err)
 		}
@@ -189,12 +199,12 @@ func (m *Roll) Complete(ctx context.Context) error {
 
 	// recreate views for the new version (if some operations require it, ie SQL)
 	if refreshViews && !m.disableVersionSchemas {
-		schema, err = m.state.ReadSchema(ctx, m.schema)
+		currentSchema, err = m.state.ReadSchema(ctx, m.schema)
 		if err != nil {
 			return fmt.Errorf("unable to read schema: %w", err)
 		}
 
-		err = m.ensureViews(ctx, schema, migration.Name)
+		err = m.ensureViews(ctx, currentSchema, migration.Name)
 		if err != nil {
 			return err
 		}
@@ -226,9 +236,15 @@ func (m *Roll) Rollback(ctx context.Context) error {
 		}
 	}
 
+	// read the current schema
+	currentSchema, err := m.state.ReadSchema(ctx, m.schema)
+	if err != nil {
+		return fmt.Errorf("unable to read schema: %w", err)
+	}
+
 	// execute operations
 	for _, op := range migration.Operations {
-		err := op.Rollback(ctx, m.pgConn, m.sqlTransformer)
+		err := op.Rollback(ctx, m.pgConn, m.sqlTransformer, currentSchema)
 		if err != nil {
 			return fmt.Errorf("unable to execute rollback operation: %w", err)
 		}
