@@ -33,6 +33,9 @@ type options struct {
 	// the duration to delay after each batch is run
 	backfillBatchDelay time.Duration
 
+	// whether to skip validation
+	skipValidation bool
+
 	migrationHooks MigrationHooks
 }
 
@@ -121,3 +124,11 @@ func WithBackfillBatchDelay(delay time.Duration) Option {
 		o.backfillBatchDelay = delay
 	}
 }
+
+// WithSkipValidation controls whether or not to perform validation on
+// migrations. If set to true, validation will be skipped.
+func WithSkipValidation(skip bool) Option {
+	return func(o *options) {
+		o.skipValidation = skip
+	}
+}
