@@ -14,10 +14,11 @@ import (
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/xataio/pgroll/internal/testutils"
 	"github.com/xataio/pgroll/pkg/migrations"
 	"github.com/xataio/pgroll/pkg/schema"
 	"github.com/xataio/pgroll/pkg/state"
-	"github.com/xataio/pgroll/pkg/testutils"
 )
 
 func TestMain(m *testing.M) {
@@ -239,7 +240,6 @@ func TestInferredMigration(t *testing.T) {
 				if err != nil {
 					t.Fatal(err)
 				}
-				defer rows.Close()
 
 				var gotMigrations []migrations.Migration
 				for rows.Next() {
@@ -253,6 +253,7 @@ func TestInferredMigration(t *testing.T) {
 					}
 					gotMigrations = append(gotMigrations, gotMigration)
 				}
+				assert.NoError(t, rows.Err())
 
 				assert.Equal(t, len(tt.wantMigrations), len(gotMigrations), "unexpected number of migrations")
 
@@ -295,7 +296,6 @@ func TestInferredMigrationsInTransactionHaveDifferentTimestamps(t *testing.T) {
 		if err != nil {
 			t.Fatal(err)
 		}
-		defer rows.Close()
 
 		type m struct {
 			name      string
@@ -312,6 +312,7 @@ func TestInferredMigrationsInTransactionHaveDifferentTimestamps(t *testing.T) {
 
 			migrations = append(migrations, migration)
 		}
+		assert.NoError(t, rows.Err())
 
 		// Ensure that the two inferred migrations have different timestamps
 		assert.Equal(t, 2, len(migrations), "unexpected number of migrations")
@@ -440,9 +441,11 @@ func TestReadSchema(t *testing.T) {
 							PrimaryKey: []string{},
 							Indexes: map[string]schema.Index{
 								"id_unique": {
-									Name:    "id_unique",
-									Unique:  true,
-									Columns: []string{"id"},
+									Name:       "id_unique",
+									Unique:     true,
+									Columns:    []string{"id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX id_unique ON public.table1 USING btree (id)",
 								},
 							},
 							CheckConstraints: map[string]schema.CheckConstraint{},
@@ -480,9 +483,11 @@ func TestReadSchema(t *testing.T) {
 							PrimaryKey: []string{},
 							Indexes: map[string]schema.Index{
 								"idx_name": {
-									Name:    "idx_name",
-									Unique:  false,
-									Columns: []string{"name"},
+									Name:       "idx_name",
+									Unique:     false,
+									Columns:    []string{"name"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE INDEX idx_name ON public.table1 USING btree (name)",
 								},
 							},
 							CheckConstraints:  map[string]schema.CheckConstraint{},
@@ -511,9 +516,11 @@ func TestReadSchema(t *testing.T) {
 							PrimaryKey: []string{"id"},
 							Indexes: map[string]schema.Index{
 								"table1_pkey": {
-									Name:    "table1_pkey",
-									Unique:  true,
-									Columns: []string{"id"},
+									Name:       "table1_pkey",
+									Unique:     true,
+									Columns:    []string{"id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
 								},
 							},
 							CheckConstraints:  map[string]schema.CheckConstraint{},
@@ -565,9 +572,11 @@ func TestReadSchema(t *testing.T) {
 							PrimaryKey: []string{"id"},
 							Indexes: map[string]schema.Index{
 								"table1_pkey": {
-									Name:    "table1_pkey",
-									Unique:  true,
-									Columns: []string{"id"},
+									Name:       "table1_pkey",
+									Unique:     true,
+									Columns:    []string{"id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
 								},
 							},
 							CheckConstraints:  map[string]schema.CheckConstraint{},
@@ -624,9 +633,11 @@ func TestReadSchema(t *testing.T) {
 							PrimaryKey: []string{"id"},
 							Indexes: map[string]schema.Index{
 								"table1_pkey": {
-									Name:    "table1_pkey",
-									Unique:  true,
-									Columns: []string{"id"},
+									Name:       "table1_pkey",
+									Unique:     true,
+									Columns:    []string{"id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
 								},
 							},
 							ForeignKeys: map[string]schema.ForeignKey{},
@@ -667,14 +678,18 @@ func TestReadSchema(t *testing.T) {
 							PrimaryKey: []string{"id"},
 							Indexes: map[string]schema.Index{
 								"table1_pkey": {
-									Name:    "table1_pkey",
-									Unique:  true,
-									Columns: []string{"id"},
+									Name:       "table1_pkey",
+									Unique:     true,
+									Columns:    []string{"id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
 								},
 								"name_unique": {
-									Name:    "name_unique",
-									Unique:  true,
-									Columns: []string{"name"},
+									Name:       "name_unique",
+									Unique:     true,
+									Columns:    []string{"name"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX name_unique ON public.table1 USING btree (name)",
 								},
 							},
 							ForeignKeys:      map[string]schema.ForeignKey{},
@@ -714,14 +729,18 @@ func TestReadSchema(t *testing.T) {
 							PrimaryKey: []string{"id"},
 							Indexes: map[string]schema.Index{
 								"table1_pkey": {
-									Name:    "table1_pkey",
-									Unique:  true,
-									Columns: []string{"id"},
+									Name:       "table1_pkey",
+									Unique:     true,
+									Columns:    []string{"id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
 								},
 								"name_id_unique": {
-									Name:    "name_id_unique",
-									Unique:  true,
-									Columns: []string{"id", "name"},
+									Name:       "name_id_unique",
+									Unique:     true,
+									Columns:    []string{"id", "name"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX name_id_unique ON public.table1 USING btree (id, name)",
 								},
 							},
 							ForeignKeys:      map[string]schema.ForeignKey{},
@@ -736,6 +755,79 @@ func TestReadSchema(t *testing.T) {
 					},
 				},
 			},
+			{
+				name: "multicolumn foreign key constraint",
+				createStmt: `CREATE TABLE products(
+          customer_id INT NOT NULL, 
+          product_id INT NOT NULL, 
+          PRIMARY KEY(customer_id, product_id));
+
+          CREATE TABLE orders(
+            customer_id INT NOT NULL, 
+            product_id INT NOT NULL, 
+            CONSTRAINT fk_customer_product FOREIGN KEY (customer_id, product_id) REFERENCES products (customer_id, product_id));`,
+				wantSchema: &schema.Schema{
+					Name: "public",
+					Tables: map[string]schema.Table{
+						"products": {
+							Name: "products",
+							Columns: map[string]schema.Column{
+								"customer_id": {
+									Name:     "customer_id",
+									Type:     "integer",
+									Nullable: false,
+								},
+								"product_id": {
+									Name:     "product_id",
+									Type:     "integer",
+									Nullable: false,
+								},
+							},
+							PrimaryKey: []string{"customer_id", "product_id"},
+							Indexes: map[string]schema.Index{
+								"products_pkey": {
+									Name:       "products_pkey",
+									Unique:     true,
+									Columns:    []string{"customer_id", "product_id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX products_pkey ON public.products USING btree (customer_id, product_id)",
+								},
+							},
+							ForeignKeys:       map[string]schema.ForeignKey{},
+							CheckConstraints:  map[string]schema.CheckConstraint{},
+							UniqueConstraints: map[string]schema.UniqueConstraint{},
+						},
+						"orders": {
+							Name: "orders",
+							Columns: map[string]schema.Column{
+								"customer_id": {
+									Name:     "customer_id",
+									Type:     "integer",
+									Nullable: false,
+								},
+								"product_id": {
+									Name:     "product_id",
+									Type:     "integer",
+									Nullable: false,
+								},
+							},
+							PrimaryKey: []string{},
+							Indexes:    map[string]schema.Index{},
+							ForeignKeys: map[string]schema.ForeignKey{
+								"fk_customer_product": {
+									Name:              "fk_customer_product",
+									Columns:           []string{"customer_id", "product_id"},
+									ReferencedTable:   "products",
+									ReferencedColumns: []string{"customer_id", "product_id"},
+									OnDelete:          "NO ACTION",
+								},
+							},
+							CheckConstraints:  map[string]schema.CheckConstraint{},
+							UniqueConstraints: map[string]schema.UniqueConstraint{},
+						},
+					},
+				},
+			},
 			{
 				name:       "multi-column index",
 				createStmt: "CREATE TABLE public.table1 (a text, b text); CREATE INDEX idx_ab ON public.table1 (a, b);",
@@ -759,9 +851,11 @@ func TestReadSchema(t *testing.T) {
 							PrimaryKey: []string{},
 							Indexes: map[string]schema.Index{
 								"idx_ab": {
-									Name:    "idx_ab",
-									Unique:  false,
-									Columns: []string{"a", "b"},
+									Name:       "idx_ab",
+									Unique:     false,
+									Columns:    []string{"a", "b"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE INDEX idx_ab ON public.table1 USING btree (a, b)",
 								},
 							},
 							ForeignKeys:       map[string]schema.ForeignKey{},
@@ -795,6 +889,36 @@ func TestReadSchema(t *testing.T) {
 					},
 				},
 			},
+			{
+				name:       "custom enum types",
+				createStmt: "CREATE TYPE review AS ENUM ('good', 'bad', 'ugly'); CREATE TABLE public.table1 (name text, review review);",
+				wantSchema: &schema.Schema{
+					Name: "public",
+					Tables: map[string]schema.Table{
+						"table1": {
+							Name: "table1",
+							Columns: map[string]schema.Column{
+								"name": {
+									Name:     "name",
+									Type:     "text",
+									Nullable: true,
+								},
+								"review": {
+									Name:       "review",
+									Type:       "public.review",
+									Nullable:   true,
+									EnumValues: []string{"good", "bad", "ugly"},
+								},
+							},
+							PrimaryKey:        []string{},
+							Indexes:           map[string]schema.Index{},
+							ForeignKeys:       map[string]schema.ForeignKey{},
+							CheckConstraints:  map[string]schema.CheckConstraint{},
+							UniqueConstraints: map[string]schema.UniqueConstraint{},
+						},
+					},
+				},
+			},
 		}
 
 		for _, tt := range tests {
